#ifndef STATUSWIDGET_H
#define STATUSWIDGET_H

#include <QWidget>
#include <QSettings>

namespace Ui {
class StatusWidget;
}

class PatchDownloader;
class LotroDatManager;

class StatusWidget : public QWidget
{
    Q_OBJECT

public:
    explicit StatusWidget(QSettings* settings, PatchDownloader* patch_downloader, LotroDatManager* lotro_dat_manager, QWidget *parent = 0);
    ~StatusWidget();

public slots:
    void updateFontsSizes();

protected:
    void resizeEvent(QResizeEvent *event) override;

private slots:
    void onPatchDownloaderStarted();
    void onPatchDownloaderFinished();
    void onPatchDownloaderProgressChanged(quint64 bytesDownloaded, quint64 bytesTotal,
                                          QString download_speed_formatted,
                                          QString elapsed_time_formatted);

    void on_game_button_clicked();

    void setToolTipToWeeklyCodeHelp();
    void setToolTipToWeeklyCodeComplete();
    void setToolTipToNewsHelp();
    void resetToolTip();



private:
    Ui::StatusWidget *ui;

    QSettings* app_settings;
    PatchDownloader* patch_updater;
    LotroDatManager* lotro_manager;

    const QColor inWorkColor = QColor(85, 170, 255);
    const QColor readyColor = QColor(0, 170, 0);
    const QColor errorColor = QColor(255, 85, 0);

    const QString inWorkIconStyle = "image: url(:/assets/info.png);";
    const QString readyIconStyle = "image: url(:/assets/ok.png);";
    const QString criticalIconStyle = "image: url(:/assets/critical.png);";
    const QString warningIconStyle = "image: url(:/assets/warning.png);";
};

#endif // STATUSWIDGET_H
